#!/usr/bin/python -t
# -*- coding: UTF8 -*-

## Copyright (C) 2007 Red Hat Inc.
## Written by Barry <mbarabas@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# import necessary libraries
import os, sys, commands, time, re
import pygtk
import gnome, gnome.ui
pygtk.require( "2.0" )

try:
    import gtk
except RuntimeError, e:
    print "This is a graphical application and requires DISPLAY to be set."
    sys.exit (1)


from scheduler.i18n import _
from scheduler import scheduler
from scheduler import errors
from guiRender import RenderObjects
from guiOutput import OutputFormatter
from sectool   import print_exc_plus

import threading
import time

# global variables
APP_VERSION = "0.9.1"

PATH_APPICON = '/usr/share/pixmaps/sectool-gui.png'     #: path to application icon
PATH_APPCONF = '/etc/sectool/sectool.conf'              #: path to configuration file

TEST_COLUMN = 0                                         #: which column is the name of test (in testView)
FILTER_NONE_POS = 0                                     #: which position of combo box (group filter) is "All" (default)
HINT_SHIFT_X = 30                                       #: horizontal shift of hint from the mouse pointer 
HINT_SHIFT_Y = 50                                       #: vertical shift of hint from the mouse pointer
PROGRESS_WIDTH = 300                                    #: width of progress bar
PROGRESS_HEIGHT = 30                                    #: height of progress bar
DEFAULT_SELECTED_LEVEL = 1                              #: which level will be selected as default


class Handler( object ):

    def init_handlers( self ):
        """ Initialize handlers for current object"""
        raise "Module "+self.__str__+" has no \"init_handlers\" method"


class SecToolGUI(threading.Thread):

    """
    Main Class of project. Class initializes the root window, handler class and run application
    """

    def __init__( self ):
        """
        Initial method on main class. 
        """

        threading.Thread.__init__(self)
        self.version      = APP_VERSION
        self.program_name = _("Security Tool GUI")
        
        gnome.init( self.program_name, self.version )
        
        # initializing main window
        self.mainWindow = gtk.Window( gtk.WINDOW_TOPLEVEL )
        self.mainWindow.set_title(_("Security Tool"))
        self.mainWindow.resize(800, 600)
        self.mainWindow.set_icon_from_file( PATH_APPICON )
        
        self.__init_handlers( )
        

    def __init_handlers( self ):
        """
        Initialize signal handler object for all signals from widgets
        This method is called directly after __init__ function
        """

        self.signalHandler = SignalHandler( )
        self.testHandler = TestHandler( )
        self.formatter = OutputFormatter( )
        self.renderer = RenderObjects( self.signalHandler, self.formatter )

        # inint handlers in all objects
        self.signalHandler.init_handlers( self.testHandler, self.formatter, self.mainWindow )
        self.testHandler.init_handlers( self.signalHandler, self.formatter )
        self.formatter.init_handlers( self.renderer )
        
    def run( self ):
        """
        Function to run program
        """
        # render content of mainWindow
        
        self.mainWindow.show( )
        # this line below have to be here ! else pane is *****
        self.formatter.renderApp( self.mainWindow )
        self.testHandler.init_tests( )
        self.testHandler.init_groups( )
        self.testHandler.set_default_level( )
        # select first test on start
        self.formatter.select_test( 0 )
        
        # start main loop
        gtk.main( )


class SignalHandler( Handler ):

    """
    Class which is taking care about all callback staff. Here are all of the application functions 
    """

    
    def __init__( self ):
        """ 
        Here come some init staff 
        """
        
        self.__mouseX = 0                         #: initial value of mouse position
        self.__mouseY = 0                         #: initial value of mouse position
        self.__selected_page = 0                  #: default selected page
        self.__level = DEFAULT_SELECTED_LEVEL     #: default test level
        
        # array of signals.. please make the changes only in this array, keys are used to search for methodes
        self.__signals = {
                        # window with hint is closed
                        "HINT_WINDOW_CLOSE" : self.__window_state_changed,
                        # window with hint is moved
                        "WINDOW_MOVE"  : self.__window_move,
                        # user choose a test
                        "TESTVIEW_TOGGLED": self.__testView_toggled,
                        # check some of the level checkButtons in testView
                        "TESTVIEW_LEVEL_TOGGLED" : self.__testView_level_toggled,
                        # user clicked with button on the test (maybe popup?)
                        "TESTVIEW_BUTTON_PRESSED": self.__testView_button_pressed,
                        # starting custom level 
                        "START_TEST": self.__start_test,
                        # starting tests on level
                        "START_LEVEL": self.__start_level,
                        # ending program
                        "QUIT": self.__destroy,
                        # about dialog
                        "ABOUT_DIALOG": self.__about_dialog,
                        # hint toggled
                        "HINT_TOGGLED": self.__menuViewHint_toggled,
                        # hint toggled
                        "INFO_TOGGLED": self.__menuViewInfo_toggled,
                        # switched page on notebook
                        "NB_SWITCH_PAGE": self.__mainBook_switch_page,
                        # toggled messages in outputView
                        "OUTPUT_TOGGLED": self.__outputView_toggled,
                        # toggled comboBox with group filter
                        "FILTER_GROUP": self.__filter_group,
                        # toggled comboBox with level filter
                        "FILTER_LEVEL": self.__filter_level,
                        # toggled comboBox with level filter
                        "STOP_TESTS": self.__stop_tests,
                        # when mainWindow moved
                        "WINDOW_MOVE":self.__window_move,
                        # when mainWindow resized
                        "WINDOW_RESIZE": self.__window_resize,
                        # window changed state event
                        "WINDOW_STATE_CHANGED" : self.__window_state_changed,
                        # mainWindow lost focus
                        "WINDOW_FOCUS_OUT": self.__window_state_changed,
                        # set hiding mode for progress bar
                        "PROGRESSBAR_SET_HIDE": self.__progressButton_toggled,
                        # save button in toolBar was toggled
                        "SAVE_OUTPUT": self.__save_output_toggled,
                        # toggled test in result page in test view
                        "OUTPUT_TEST_TOGGLED": self.__outputtestView_toggled,
                        # load default level configuration
                        "DEFAULT_LEVELS": self.__load_default_levels,
                        # refresh test data
                        "REFRESH_TEST": self.__refresh_test,
                        # refresh test data
                        "REFRESH_TEST_ALL": self.__refresh_test_all,
                        }

    def get_callback( self, signal ):
        """ Function to get callback belongs to signal """
        
        if signal in self.__signals: return self.__signals[ signal ]
        else: return self.__empty

    def __empty(self, *args, **kwargs):
        """ Empty function. it returns get_callback func. if signal not in dict. signals """
        pass

    def init_handlers( self, testHandler, formatter, mainWindow ):
        """ Initialize signal handlers for mainWindow and some init staff """
        
        self.__mainWindow = mainWindow
        self.__testHandler = testHandler
        self.__formatter = formatter

        # connect basic signals of mainWindow
        self.__mainWindow.connect( "destroy", self.get_callback("QUIT") )
        self.__mainWindow.connect( "configure-event", self.get_callback("WINDOW_MOVE") )
        self.__mainWindow.connect( "window-state-event", self.get_callback("WINDOW_STATE_CHANGED") )
        self.__mainWindow.connect( "focus-out-event", self.get_callback("WINDOW_FOCUS_OUT") )

    def __start_test( self, widget ):
        """ The button to start chosen tests was pressed """
        
        #clear section
        self.__formatter.clear_output( )
        self.__formatter.hintWindow_hide( )
        # get which tests should be started
        self.test_list = self.__formatter.get_selected_tests()
        # get names from test model for every selected test
        
        if len(self.test_list) > 0:
            self.__testHandler.runtests( self.test_list, self.__level )
            if self.__selected_page != 1:
                self.__formatter.highlight_page( 1, True )
        
    def __start_level( self, widget ):
        """ The button to start whole level was pressed """
        
        #clear section
        self.__formatter.clear_output( )
        self.__formatter.hintWindow_hide( )
        # get name for active test
        self.__level = self.__formatter.get_active_level()
        self.test_list = [ test[TEST_COLUMN] for test in self.__formatter.get_active_tests(self.__level)]
        if len(self.test_list) > 0:
            # run all selected tests
            self.__testHandler.runtests( self.test_list, self.__level )
            if self.__selected_page != 1:
                self.__formatter.highlight_page( 1, True ) # make page bold if it's not selected

    def __stop_tests( self, widget ):
        """ The button to stop all testing progress was pressed """
        
        self.__testHandler.stop_tests()
        self.__formatter.BtnEvent_stop_pressed()
        self.__formatter.updateProgressBar( label=_("Cancelling, please wait...") )

    def __refresh_test( self, widget ):
        """ Refresh selected test's data """
        
        #clear section
        self.__formatter.clear_output( )
        self.__formatter.hintWindow_hide( )
        # get which tests should be started
        self.test_list = self.__formatter.get_selected_tests()
        # get names from test model for every selected test
        level = -1 # refreshing 
        if len(self.test_list) > 0:
            self.__testHandler.runtests( self.test_list, level )

    def __refresh_test_all( self, widget ):
        """ Refresh selected test's data """
        
        #clear section
        self.__formatter.clear_output( )
        self.__formatter.hintWindow_hide( )
        # get name for active test
        level = -1
        self.test_list = [ test[TEST_COLUMN] for test in self.__formatter.get_all_tests()]
        if len(self.test_list) > 0:
            # run all selected tests
            self.__testHandler.runtests( self.test_list, level )
            

    def __about_dialog( self, widget ):
        """ About dialog .. classic staff. """

        about = gnome.ui.About(_("Fedora Security Tool"),
                               _("Copyright by Red Hat "),
                               _("An auditing tool for Fedora and Red Hat systems only."),
                               _("Version") + " " + APP_VERSION,
                                ["Peter Vrabec <pvrabec@redhat.com>", 
                                "Jakub Hrozek <jhrozek@redhat.com>",
                                "Maros Barabas <mbarabas@redhat.com>", 
                                "Michel Samia <msamia@redhat.com",
                                "Daniel Kopecek <dkopecek@redhat.com"
                                ])
        about.show()

    def __filter_group( self, widget, combo ):
        """ Toggled comboBox which is filtering the tests in testView by group membership """

        if combo.get_active( ) == FILTER_NONE_POS: 
            self.__testHandler.filter_tests(None)
        else: self.__testHandler.filter_tests( combo.get_active_text() )

    def __filter_level( self, widget, combo ):
        """ Toggled comboBox which is filtering the tests in testView by level membership """

        self.__level = combo.get_active( )+1
        self.__formatter.highlight_level( self.__level )

    def __outputtestView_toggled( self, widget, treeView ):
        """ The test in output view is chosen """
        
        self.__formatter.hintWindow_hide( )
        selection = treeView.get_selection( )
        if selection != None: (model, iter) = selection.get_selected( )
        else: iter = None

        if iter != None:
            self.__formatter.refilter_messages( )
            self.__formatter.show_test_summary( model.get_value( iter, TEST_COLUMN ) )
    
    def __mainBook_switch_page( self, widget, page, page_num ):
        """ The notebook page was switched """
        
        self.__selected_page = page_num
        self.__formatter.hintWindow_hide( )
        self.__formatter.highlight_page( page_num, False )

    def __outputView_toggled( self, widget, treeView=None ):
        """ One message in output-view is toggled"""
        
        selection = treeView.get_selection( )
        if selection: 
            (model, iter) = selection.get_selected( )
        else: iter = None

        if iter != None:
            # compute the position of window and set the current position of mouse 
            self.__mouseX = self.__mainWindow.get_pointer()[0]
            self.__mouseY = self.__mainWindow.get_pointer()[1]
            self.__formatter.hintWindow_move( self.__mainWindow.get_position()[0] + int(self.__mouseX) + HINT_SHIFT_X, 
                                self.__mainWindow.get_position()[1] + int(self.__mouseY) + HINT_SHIFT_Y )
            self.__testHandler.show_message_hint( model.get_value( iter, 2 ), model.get_value( iter, 3 ) )

    def __testView_toggled( self, widget, treeView ):
        """ The test is chosen """
        
        self.__formatter.hintWindow_hide( )
        selection = treeView.get_selection( )
        if selection != None: (model, iter) = selection.get_selected( )
        else: iter = None

        if iter != None:
            # show info about selected test
            self.__testHandler.show_test_info( model.get_value( iter, TEST_COLUMN ) )

    def __testView_level_toggled( self, widget, path, model, id ):
        """ One checkbox in test-view is toggled """

        if model[path][0][0] != '<':
            model[path][id] = not model[path][id] # make check-box checked or unchecked

    def __testView_button_pressed(self, widget, event, treeView):
        """ User pressed a button on the testView """

        if event.button == 3:
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = treeView.get_path_at_pos(x,y)
            if pthinfo is not None:
                path,col,cellx,celly = pthinfo
                treeView.grab_focus()
                treeView.set_cursor(path,col,0)
                widget.popup(None,None,None,event.button,event.time)

                widget.popup(None, None, None, event.button, event.time)

    def __menuViewHint_toggled( self, widget ):
        """ menu -> View -> Show hints was toggled """
        
        self.__formatter.BtnEvent_hint_toggle( widget.get_active( ) )
        if widget.get_active( ) and len( self.__formatter.hint_msg ) > 0 :
            self.__formatter.hintWindow_show( )
        else: 
            self.__formatter.hintWindow_hide( )

    def __menuViewInfo_toggled( self, widget ):
        """ menu -> View -> Show info messages  was toggled """
        
        self.__formatter.refilter_messages( )
        self.__formatter.BtnEvent_info_toggle( widget.get_active() )

    def __window_resize( self, widget, data ):
        """ When window is resized """
        # we need to resize all objects depending on window size
        self.__formatter.MainWindow_resized( self.__mainWindow.get_size()[0], self.__mainWindow.get_size()[1] )

    def __window_move( self, widget, data ):
        """ When window is dragged and moved"""
        
        self.__window_resize( None, None )
        # compute the new state of hintWindow
        self.__formatter.hintWindow_move( self.__mainWindow.get_position()[0] + int(self.__mouseX) + HINT_SHIFT_X, 
                              self.__mainWindow.get_position()[1] + int(self.__mouseY) + HINT_SHIFT_Y )

    def __window_state_changed( self, widget, event ):
        """ When window is closed"""
        
        self.__formatter.hintWindow_hide( )
        self.__window_resize( None, None )

    def __progressButton_toggled( self, widget ):
        """ Progress button clicked """
        if not widget.get_active(): self.__formatter.hide_progressBar( )

    def __load_default_levels( self, widget ):
        """ Load default levels for levels configuration for every test """
        defaultTestsLevels = self.__testHandler.get_default_levels( )

        for test in defaultTestsLevels:
            self.__formatter.set_test_levels( test, defaultTestsLevels[ test ] )

    def __destroy( self, widget ):
        """ Self destroy - make as much damage as possible """
        
        self.kill_thread( )

        #: get changes in test configuration if there is a change save it.
        save_tests = self.__testHandler.get_levels_diff( )
        if save_tests != {}:
            # get some dialog
            if self.__formatter.save_box( ):
                self.__testHandler.save_levels( save_tests )

        gtk.main_quit( )
        sys.exit()

    def __save_output_toggled( self, widget ):
        """ Save button clicked """
        self.__formatter.save_to_file( )

    def kill_thread( self ):
	""" 
	Called when stop button is pressed or application is going to be destroyed
	Method is using command ps to check python processes with PID file (current process) in output:

		>>> commands.getoutput('ps -eLcf | grep '+`pid`+' | grep -v "ps -eLcf"')

	After that it kills every process wich is child of current process, if there is such process
	"""

        pid = 1
        oldpid = 0
        iteration = 0
        while pid != 0:
            if oldpid == pid: time.sleep(0.1) #: we are trying to kill the same process again, just wait 100ms, maybe we are too fast
            if iteration > 10: 
                print "Thread(%s) can't be killed too much iterations !" % (pid) #: trying to kill one thread over 10 times (1s) - is too much
                break
            pid = os.getpid()
            processes_text = commands.getoutput('ps -eLcf | grep '+`pid`+' | grep -v "ps -eLcf"') #: get the list of running threads
            processes = processes_text.split('\n')
            ppid_list = [ process.split()[1] for process in processes ]
            pid = 0
            for process in processes:
                if process.split()[2] in ppid_list:
                    pid = process.split()[1]
            if pid != 0:
                oldpid = pid
                iteration += 1
                output = commands.getoutput( 'kill -9 ' + `pid` )
                if output != '': print "Thread cann't be killed: " + output
        

class TestHandler( Handler ): 
    """
    Class taking care of tests
    """
    
    def __init__( self ):
        """ Load necessary staff """
        
        #: load config options from the scheduler
        try:
            self.scheduler_conf = scheduler.SecToolConf( PATH_APPCONF )
            self.ST_TestLoader = scheduler.ST_TestLoader( level_config=os.path.join(os.path.expanduser("~"), self.scheduler_conf.userfile) )
            self.scheduler_conf.validate()
        except Exception, e:
            print_exc_plus() #: if this line is an exception message probably the loading of environ-variables below is crashing
            raise str(e)
        
        self.environ = dict([ (key.upper(), self.scheduler_conf['PATHS'][key]) for key in self.scheduler_conf['PATHS'] ])
        self.environ.update( dict([ (key.upper(), self.scheduler_conf['ENVIRONMENT'][key]) for key in self.scheduler_conf['ENVIRONMENT'] ]) )
        self.environ.update( { 'TDATA_DIR_BASE' : self.environ['TDATA_DIR_BASE'] } )
        self.environ.update( { 'PATH' : self.environ['PATH'] } )
        self.environ.update( DEBUG='0' )
        self.suitesAll = self.ST_TestLoader.loadFromPath( self.environ["DSC_DIR"], self.environ["TESTS_DIRS"] )
            
    def __remove_mark( self, string ):
        
        if string == None: return ""
        str = re.search( ">.*<", string ) 
        if str != None: 
            #print str.group(0)[1:-1]
            return str.group(0)[1:-1]
        else: return string

    def set_environ( REFRESH=None, LEVEL=None, DEBUG=None ):

        if REFRESH != None: self.environ.update( REFRESH = REFRESH )
        if DEBUG != None: self.environ.update( DEBUG = DEBUG )
        if LEVEL != None: self.environ.update( LEVEL = LEVEL )

    def set_default_level( self ):

        self.__formatter.select_level( self.scheduler_conf['ACTION']['LEVEL'] )

    def init_handlers( self, signalHandler, formatter ):
        """ Initialize signal handler object for all signals from widgets """
        
        self.__signalHandler = signalHandler
        self.__formatter = formatter

    @staticmethod
    def filterGroup( dsc, group ):
        """ Filter only test-cases that are in the group """
        
        return group in dsc["HEADER"]["GROUPS"]

    @staticmethod
    def filterTestByName( dsc, name ):
        """ Return test-case with this name """
        
        return name in dsc["HEADER"]["NAME"]

    @staticmethod    
    def filterTestNames( dsc, test_list ):
        """ Filter test-cases by names """
        
        names = set( dsc["HEADER"]["NAME"].split() )
        return (names & set(test_list)) != set([])

    def filter_tests( self, filter):
        """ Function filtering tests by groupBox in simpleFilter """
        
        self.__formatter.clear_testView() #: clear testView to make a new list
        # filter tests by selected group or show all tests
        if filter != None:
            suites = self.suitesAll.filter( self.filterGroup, group = filter )
        else: suites = self.suitesAll 
        
        # Fill testView with test-names defines in levels from suites
        for suite in suites:
           levels = suite.valid_levels
           self.__formatter.add_test([suite.dsc["HEADER"]["NAME"], (False, True)[1 in levels], (False, True)[2 in levels], (False, True)[3 in levels], 
                                           (False, True)[4 in levels], (False, True)[5 in levels], True], suite.check_requires( suite.dsc )+suite.checkForDeps( suite.dsc ) )
        
    def init_groups( self ):
        """ Filled group_filter combo box with groups """
        
        tmpGroups = []
        for suite in self.suitesAll:
            for group in suite.dsc["HEADER"]["GROUPS"].split( " " ):
               if group not in tmpGroups:
                   tmpGroups.append( group )
                   self.__formatter.add_group( group )
                    
    def init_tests( self ):
        """ Filled testView """
        # this function is only for abstract initializing testView and filtering testView, do not remove it !!
        self.filter_tests( None )
                
    def runtests( self, test_list, level ):
        """ Run selected test(s) in new thread in specified level """
        for e,test in enumerate(test_list):
            test_list[e] = self.__remove_mark( test )
            
        suites = self.suitesAll.filter( self.filterTestNames, test_list=test_list )

        suites_updated = []
        test_list = []
        for suite in suites: 
            if len( suite.check_requires( suite.dsc )+suite.checkForDeps( suite.dsc ) ) == 0:
                suites_updated.append( suite )
                test_list.append( suite.dsc["HEADER"]["NAME"] )
                
        if len( test_list ) == 0: return
        suite = self.suitesAll.filter( self.filterTestNames, test_list=test_list )
        if level == -1: self.environ.update( REFRESH = '1' )
        else: self.environ.update( REFRESH = '0' )
        suite.updateEnviron(self.environ)
        
        self.threadHandler = ThreadHandler( self.__formatter, suite, test_list, level ) #: run tests in new thread
        self.threadHandler.start()
    
    def stop_tests( self ):
        """ Stop button has been pressed """
        
        self.threadHandler.stop() #: give a short amount of time to stop the test by itself
        self.__signalHandler.kill_thread() #: kill the test by unix-style kill through os library

    def show_message_hint( self, test, msg_id ):
        """ Show info message in hintWindow """
        # this is for case to make some test-like changes before adding the message information
        self.__formatter.show_message_hint( test, msg_id ) 

    def show_test_info( self, test ):
        """ Show info about selected test i mainWindow """
        
        suites = self.suitesAll.filter( self.filterTestByName, name=self.__remove_mark( test ) )
        for suite in suites: pass #: get the last suite .. maybe TODO - suites[-1] ?
        self.__formatter.show_test_info( suite.dsc["HEADER"]["NAME"], 
                                         suite.dsc["HEADER"]["AUTHOR"], 
                                         suite.dsc["HEADER"]["DESCRIPTION"], 
                                         suite.dsc["HEADER"]["GROUPS"], 
                                         suite.valid_levels, #suite.dsc["HEADER"]["LEVELS"], 
                                         suite.check_requires( suite.dsc )+suite.checkForDeps( suite.dsc ) )

    def save_levels( self, save_tests ):

        for test in save_tests:
            self.ST_TestLoader.level_override[ test ] = save_tests[ test ]

        self.scheduler_conf.save_custom_levels( self.ST_TestLoader.level_override )

    def get_levels_diff( self ):
        
        userTestsLevels = self.__formatter.get_tests_levels( )
        udefaultTestsLevels = self.__get_user_levels( )

        save_tests = dict()

        for test in udefaultTestsLevels:
            if test in userTestsLevels and udefaultTestsLevels[ test ] != userTestsLevels[ test ]:
                save_tests[ test ] = userTestsLevels[ test ]

        return save_tests

    def __get_user_levels( self ):
        """ Get default levels for all tests in suite """
        testSuite = dict()
        
        for tcase in self.suitesAll:
            testSuite[ tcase.dsc["HEADER"]["NAME"] ] = []
            for level in tcase.valid_levels :
                if level != 0: testSuite[ tcase.dsc["HEADER"]["NAME"] ].append( level )

        return testSuite

    def get_default_levels( self ):
        """ Get default levels from dscs """

        testSuite = dict()
        
        for tcase in self.suitesAll:
            testSuite[ tcase.dsc["HEADER"]["NAME"] ] = []
            levels = [ int(level) for level in tcase.dsc["HEADER"]["LEVELS"].split() ]
            for level in levels :
                if level != 0: testSuite[ tcase.dsc["HEADER"]["NAME"] ].append( level )

        return testSuite


class ThreadHandler(threading.Thread, Handler):
    """
    Thread worker used by testHandler to make new thread when executing suite of tests
    """
    
    def __init__( self, formatter, suite, test_list, level ):
        """ Initializing variables """
        
        self.__formatter = formatter
        self.__suite = suite
        self.__test_list = test_list
        self.__level = level

        threading.Thread.__init__(self)
        self.__stopthread = threading.Event()
    
    @staticmethod
    def fill_None( line, self ):
        
        pass
    
    @staticmethod
    def fill_form( line, self ):

        gtk.gdk.threads_enter()
        self.__formatter.showTestLine( self.test, line )
        gtk.gdk.threads_leave()

    def run( self ):
        """ Run method, this is the code that runs while thread is alive """
        
        step = (100/len(self.__test_list))/100.0
        done = 0.0
        
        # clear battleground before starting the test
        self.__formatter.show_progress( )
        self.__formatter.clear_results( )
        self.__formatter.set_btn_sensitivity( True )
        self.__formatter.updateProgressBar( done*step, _("Initializing..") )

        # for all tests in suite run test
        if self.__level == -1: # refreshing - quick solution
            
            for test in self.__suite:
                
                if self.__stopthread.isSet(): break
                self.__formatter.showRefreshTest( test )
                self.test = test
                test.register_callback( self.fill_None, self )
                if len( test.check_requires( test.dsc ) + test.checkForDeps( test.dsc ) ) != 0:
                    continue
                gtk.gdk.threads_enter()
                self.__formatter.updateProgressBar( done*step, test.dsc['HEADER']['NAME'] )
                gtk.gdk.threads_leave()
                
                try:
                    test.refreshTest()
                except IOError, strerror: 
                    print "%s -> skipping" % (strerror)
                    continue
                except KeyError, strerror:
                    print "%s -> skipping" % (strerror)
                    continue
                except errors.MissingDependencyError, strerror:
                    print "%s -> skipping" % (strerror)
                    continue
                except errors.MissingRequirementError, strerror:
                    print "%s -> skipping" % (strerror)
                    continue
                done += 1
                gtk.gdk.threads_enter()
                self.__formatter.showRefreshResult( test )
                gtk.gdk.threads_leave()
                #self.__formatter.show_test_summary( test.dsc['HEADER']['NAME'] )
                # TODO: update selected test
            
        else:

            for test in self.__suite:
    
                if self.__stopthread.isSet(): break
                self.__formatter.showTest( test )
                self.test = test
                test.register_callback( self.fill_form, self )
                if len( test.check_requires( test.dsc ) + test.checkForDeps( test.dsc ) ) != 0:
                    continue
    
                time_start = time.time() #: set timer to know when was test started
                gtk.gdk.threads_enter()
                self.__formatter.updateProgressBar( done*step, test.dsc['HEADER']['NAME'] )
                gtk.gdk.threads_leave()
                try:
                    result = test.runTestLevel( self.__level )
                except: #: first exception is only when test is not defined in self.__level, just run it in default (0) level
                    try:
                        result = test.runTestLevel( test.default_level )
                    except IOError, strerror: 
                        print "%s -> skipping" % (strerror)
                        continue
                    except KeyError, strerror:
                        print "%s -> skipping" % (strerror)
                        continue
                    except errors.MissingDependencyError, strerror:
                        print "%s -> skipping" % (strerror)
                        continue
                    except errors.MissingRequirementError, strerror:
                        print "%s -> skipping" % (strerror)
                        continue
                done += 1
                time_end = time.time() #: stop the test timer.. make spent time to show for how long was test executed
                spent_time = time_end - time_start
                gtk.gdk.threads_enter()
                self.__formatter.showTestResult( test, result, spent_time )
                #self.__formatter.show_test_summary( test.dsc['HEADER']['NAME'] )
                # TODO: update selected test
                gtk.gdk.threads_leave()
            
        gtk.gdk.threads_enter()
        self.__formatter.updateProgressBar( 1.0, "" ) #: do not change label "" .. for changing label please go to func updateProgressBar
        self.__formatter.hide_progressBar( )          #: this function hide progressbar only in case that progressButton is not active
        self.__formatter.set_btn_sensitivity( False )
        gtk.gdk.threads_leave()
        
    def init_handlers( self ):
        pass

    def stop( self ): 
         """Stop method, sets the event to terminate the thread's main loop"""
         
         self.__stopthread.set( ) #: the testing process is going to die .. we have a little time to stop all tests correctly

        
def run():
    """
    Start main program loop and load i18n strings
    """
    
    gtk.gdk.threads_init()
    main = SecToolGUI( )
    main.start( )
    
if __name__ == "__main__":
    try:
        run()
    except KeyboardInterrupt: #: ^C .. what a noob ?!?
        print >> sys.stderr, _("Exiting on user request.\n")
        pass
